@@ -47,6 +47,18 @@ const gchar *gdu_utils_get_seat (void);
 gchar *gdu_utils_format_duration_usec (guint64                usec,
                                        GduFormatDurationFlags flags);
 
+gboolean gdu_utils_is_flash                  (UDisksDrive *drive);
+
+guint gdu_utils_count_primary_dos_partitions (UDisksClient         *client,
+                                              UDisksPartitionTable *table);
+
+gboolean gdu_utils_have_dos_extended         (UDisksClient         *client,
+                                              UDisksPartitionTable *table);
+
+gboolean gdu_utils_is_inside_dos_extended    (UDisksClient         *client,
+                                              UDisksPartitionTable *table,
+                                              guint64               offset);
+
 void            gdu_utils_show_error      (GtkWindow      *parent_window,
                                            const gchar    *message,
                                            GError         *error);
@@ -62,6 +74,34 @@ gboolean        gdu_utils_show_confirmation (GtkWindow    *parent_window,
 
 gboolean gdu_utils_is_ntfs_available (void);
 
+#ifdef HAVE_UDISKS2_7_2
+
+/* Defined by libblockdev/UDisks */
+typedef enum {
+  OFFLINE_SHRINK = 1 << 1,
+  OFFLINE_GROW = 1 << 2,
+  ONLINE_SHRINK = 1 << 3,
+  ONLINE_GROW = 1 << 4
+} ResizeFlags;
+
+gboolean gdu_utils_can_resize (UDisksClient *client,
+                               const gchar  *fstype,
+                               gboolean      flush,
+                               ResizeFlags  *mode_out,
+                               gchar       **missing_util_out);
+
+gboolean gdu_utils_can_repair (UDisksClient *client,
+                               const gchar  *fstype,
+                               gboolean      flush,
+                               gchar       **missing_util_out);
+
+gboolean gdu_utils_can_check  (UDisksClient *client,
+                               const gchar  *fstype,
+                               gboolean      flush,
+                               gchar       **missing_util_out);
+
+#endif
+
 guint gdu_utils_get_max_label_length (const gchar *fstype);
 
 gboolean _gtk_entry_buffer_truncate_bytes (GtkEntryBuffer *gtk_entry_buffer,
@@ -95,10 +135,50 @@ gboolean gdu_utils_ensure_unused_list_finish (UDisksClient  *client,
                                               GAsyncResult  *res,
                                               GError       **error);
 
+guint64 gdu_utils_calc_space_to_grow (UDisksClient *client,
+                                      UDisksPartitionTable *table,
+                                      UDisksPartition *partition);
+
+guint64 gdu_utils_calc_space_to_shrink_extended (UDisksClient *client,
+                                                 UDisksPartitionTable *table,
+                                                 UDisksPartition *partition);
+
 gint64 gdu_utils_get_unused_for_block (UDisksClient *client,
                                        UDisksBlock  *block);
 
 
+#define NUM_UNITS 11
+
+typedef enum {
+  Byte,
+  KByte,
+  MByte,
+  GByte,
+  TByte,
+  PByte,
+  KiByte,
+  MiByte,
+  GiByte,
+  TiByte,
+  PiByte
+} UnitSizeIndices;
+
+/* Keep in sync with Glade file */
+static const guint64 unit_sizes[NUM_UNITS] = {
+  (1ULL),                /*  0: bytes */
+  (1000ULL),             /*  1: kB */
+  (1000000ULL),          /*  2: MB */
+  (1000000000ULL),       /*  3: GB */
+  (1000000000000ULL),    /*  4: TB */
+  (1000000000000000ULL), /*  5: PB */
+  ((1ULL)<<10),          /*  6: KiB */
+  ((1ULL)<<20),          /*  7: MiB */
+  ((1ULL)<<30),          /*  8: GiB */
+  ((1ULL)<<40),          /*  9: TiB */
+  ((1ULL)<<50),          /* 10: PiB */
+};
+
+gint gdu_utils_get_default_unit (guint64 size);
 
 G_END_DECLS
 
